/*
=================================================
|            TPL Stepper Control V1.0ß          |
|                                               |
|        an Arduino "NodeMCU Lolin V3",         |
| Sender-/Empfaenger-Set Modell: "MX- FS- 03V"  |
|                                               |
|     WiFi, HTML & JavaScript kontrolliert      |
|  (FireFox und IE 10 mit Windows 10 getestet)  |
|                                               |
| Fuer Boardtest & zeitunkritische  Anwendungen |
| ACHTUNG:  Reaktionzeiten  des Statustwechsels |
| in BetaVersion je nach Browser/Betriebssystem |
|                   2 - 5 sec.                  |
|                                               |
|   (c) TPL Consulting, Thomas P. Ledwon 2018   |
|             www.tplconsulting.com             |
=================================================
 */

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>

#include "index.h"

//********************* On board LED ggf. aus. ************************
//SW Ueberbrueckung: INPUT-Schalter fuer A0-IN-Dauerbetrieb der Stepper
#define LED0 16 
//*********************************************************************

#define LED1 5
#define LED2 4
#define LED3 0 
#define LED4 2  //On board LED
#define LED5 14
#define LED6 12
#define LED7 13
#define LED8 15


//STEPPER 1 ************************************
//  Motorbrücke        | IN1 | IN2 | IN3 | IN4 |    letzen 4 Bits n. genutzt!
//  Board-Pin          | D-1 | D-2 | D-3 | D-4 |
//                     -------------------------
int arrayStepsLED1[] = {  1,    1,    0,    0,};         //0, 0, 0, 1};
int arrayStepsLED2[] = {  0,    1,    0,    1,};         //0, 0, 0, 0};
int arrayStepsLED3[] = {  1,    0,    0,    1,};         //1, 1, 0, 0};
int arrayStepsLED4[] = {  0,    0,    1,    0,};         //0, 1, 1, 1};
//                     -------------------------
//**********************************************


//STEPPER 2 ************************************
//  Motorbrücke        | IN1 | IN2 | IN3 | IN4 |    letzen 4 Bits n. genutzt!
//  Board-Pin          | D-5 | D-6 | D-7 | D-8 |
//                     -------------------------
int arrayStepsLED5[] = {  1,    1,    0,    0,};         //0, 0, 0, 1};
int arrayStepsLED6[] = {  0,    1,    0,    1,};         //0, 0, 0, 0};
int arrayStepsLED7[] = {  1,    0,    0,    1,};         //1, 1, 0, 0};
int arrayStepsLED8[] = {  0,    0,    1,    0,};         //0, 1, 1, 1};
//                     -------------------------
//**********************************************

int schalt; // SW-Bruecke. HE-Schalter (D0 INNPUT_PULLUP)
int brueckAnal1;
int brueckAnal2;
int delPara1;
int delPara2;

//SSID Password WiFi router
const char* ssid = "your Network Name";
const char* password = "your Password";

ESP8266WebServer server(80); //Server on port 80


//=========================================
//            Browser öffnen
//=========================================

void handleRoot() {
 String s = MAIN_page; //HTML Inhalt
 server.send(200, "text/html", s);
}

int anal1;
String adcValue1;
String ADCValue1;
void handleADC1() {
 anal1 = analogRead(A0);
 String adcValue1 = String(anal1);
 server.send(200, "text/plane", ADCValue1=adcValue1);
}

int anal2;
String adcValue2;
String ADCValue2;
void handleADC2() {
 anal2 = analogRead(A0);
 String adcValue2 = String(anal2);
 server.send(200, "text/plane", ADCValue2=adcValue2);
}


String M1HWPara1;
int schPara1;
int repPara1;
String M1dirPara1 = "leer";
String M1schaltPara1;

void handleSchieber1() {
handleADC1();
M1HWPara1 = server.arg("M1set");
M1dirPara1 = server.arg("M1dir");
String statSch1 = server.arg("schieber1");
String statRep1 = server.arg("rep1");
repPara1 = statRep1.toInt();
schPara1 = statSch1.toInt();
if(M1HWPara1 == "true"){schPara1 = anal1;}

M1schaltPara1 = server.arg("M1schalt"); 

Serial.println(" ");
Serial.println(" ");
Serial.println("*******************");
Serial.println("PARAMETER");
Serial.print("HW/SW1: ");Serial.println(M1HWPara1);
Serial.print("Rev1.:  ");Serial.println(schPara1);
Serial.print("Rep1.:  ");Serial.println(repPara1);
Serial.print("Dir1.:  ");Serial.println(M1dirPara1);
Serial.print("An/Aus1: ");Serial.println(M1schaltPara1);

server.send(200, "text/plane", statSch1);
}


String M2HWPara2;
int schPara2;
int repPara2;
String M2dirPara2 = "leer";
String M2schaltPara2;

void handleSchieber2() {
handleADC2(); 
M2HWPara2 = server.arg("M2set");
M2dirPara2 = server.arg("M2dir");
String statSch2 = server.arg("schieber2");
String statRep2 = server.arg("rep2");
repPara2 = statRep2.toInt();
schPara2 = statSch2.toInt();
if(M2HWPara2 == "true"){schPara2 = anal2;}

M2schaltPara2 = server.arg("M2schalt"); 


Serial.println(" ");
Serial.println(" ");
Serial.println("*******************");
Serial.println("PARAMETER");
Serial.print("HW/SW2: ");Serial.println(M2HWPara2);
Serial.print("Rev2.:  ");Serial.println(schPara2);
Serial.print("Rep2.:  ");Serial.println(repPara2);
Serial.print("Dir2.:  ");Serial.println(M2dirPara2);
Serial.print("An/Aus2: ");Serial.println(M2schaltPara2);

server.send(200, "text/plane", statSch2);
}

void handle1OFF() {
 String M1schalt = M1schaltPara1;
 Serial.print( M1schalt);
 if(M1schaltPara1 == "aus"){
  server.send(200, "text/plane", M1schalt);
 }
}

void handle2OFF() {
 String M2schalt = M2schaltPara2;
 Serial.print( M2schalt);
 if(M2schaltPara2 == "aus"){
  server.send(200, "text/plane", M2schalt);
 }
}
//============================================
//                  SETUP
//============================================

void setup(void)
{

  Serial.begin(115200);
  
  WiFi.begin(ssid, password); //WiFi router
  Serial.println("");

  //Warten auf Verbindung
  while (WiFi.status() != WL_CONNECTED) {
  delay(100);
  Serial.print(".");
  }

  //Port auf Ausgang
  pinMode(LED0,INPUT_PULLUP);
  pinMode(LED1,OUTPUT);
  pinMode(LED2,OUTPUT); 
  pinMode(LED3,OUTPUT); 
  pinMode(LED4,OUTPUT); //blaue LED
  pinMode(LED5,OUTPUT);
  pinMode(LED6,OUTPUT);
  pinMode(LED7,OUTPUT); 
  pinMode(LED8,OUTPUT);  

  //Serieller Monitor
  Serial.println("");
  Serial.print("Stepper-SENDER verbunden mit ");
  Serial.println(ssid);
  Serial.print("IP Adresse: ");
  Serial.println(WiFi.localIP()); 

//Routinenaufruf
  server.on("/", handleRoot);
  
  server.on("/readADC1", handleADC1);
  server.on("/readADC2", handleADC2);
 
  server.on("/setSchieber1", handleSchieber1);
  server.on("/setSchieber2", handleSchieber2);
  
  server.on("/setOFF1", handle1OFF);
  server.on("/setOFF2", handle2OFF);

//Server starten
  server.begin();

}


//===============================================
//                     LOOP
//===============================================

void loop(void) 
{
  
//Handle Client Anfrage
  server.handleClient();


//STEPPER



//****************** STEPPER 1 *********************

  if(M1schaltPara1 == "an")
  {
  for(int i = 0; i < repPara1; i++)
      {
        if(M1dirPara1 == "links"){
          
        for (int i1 = 3; i1 >= 0; i1--)  
        {
          
         //Stepper 1 Links Block
//*******
        if (arrayStepsLED2[i1] == 1)
        {
          digitalWrite(LED2, HIGH);
        }
        else
        {
          digitalWrite(LED2, LOW);
        }
//*******
        if (arrayStepsLED1[i1] == 1)
        {
          digitalWrite(LED1, HIGH);
        }
        else
        {
          digitalWrite(LED1, LOW);
        }
//*******
        if (arrayStepsLED4[i1] == 1)
        {
          digitalWrite(LED4, HIGH);
        }
        else
        {
          digitalWrite(LED4, LOW);
        }
//*******
        if (arrayStepsLED3[i1] == 1)
        {
          digitalWrite(LED3, HIGH);
        }
        else
        {
          digitalWrite(LED3, LOW);
        }
//*******
        delay(schPara1);
     }
    }
else
   {
      for (int i1 = 0; i1 < 4; i1++)
      {
        
      //Stepper 1 Rechts Block 
//*******             
        if (arrayStepsLED1[i1] == 1)
        {
          digitalWrite(LED1, HIGH);
        }
        else
        {
          digitalWrite(LED1, LOW);
        }
//*******
        if (arrayStepsLED2[i1] == 1)
        {
          digitalWrite(LED2, HIGH);
        }
        else
        {
          digitalWrite(LED2, LOW);
        }
//*******
        if (arrayStepsLED3[i1] == 1)
        {
          digitalWrite(LED3, HIGH);
        }
        else
        {
          digitalWrite(LED3, LOW);
        }
//*******
        if (arrayStepsLED4[i1] == 1)
        {
          digitalWrite(LED4, HIGH);
        }
        else
        {
          digitalWrite(LED4, LOW);
        }
//*******
        delay(schPara1);
     }
    }
    digitalWrite(LED1, LOW);
    digitalWrite(LED2, LOW);
    digitalWrite(LED3, LOW);
    digitalWrite(LED4, LOW);
    if(i == repPara1-1 && schalt == 0){
       M1schaltPara1 = "aus";
       handle1OFF();}
   }
  } 

//**************************************************


//****************** STEPPER 2 *********************

  if(M2schaltPara2 == "an")
  {
  for(int i = 0; i < repPara2; i++)
      {
        if(M2dirPara2 == "links"){
          
        for (int i1 = 3; i1 >= 0; i1--)
        {

        //Stepper 2 Links Block
        
//*******
        if (arrayStepsLED5[i1] == 1)
        {
          digitalWrite(LED5, HIGH);
        }
        else
        {
          digitalWrite(LED5, LOW);
        }
//*******
        if (arrayStepsLED6[i1] == 1)
        {
          digitalWrite(LED6, HIGH);
        }
        else
        {
          digitalWrite(LED6, LOW);
        }
//*******
        if (arrayStepsLED7[i1] == 1)
        {
          digitalWrite(LED7, HIGH);
        }
        else
        {
          digitalWrite(LED7, LOW);
        }
//*******
        if (arrayStepsLED8[i1] == 1)
        {
          digitalWrite(LED8, HIGH);
        }
        else
        {
          digitalWrite(LED8, LOW);
        }
//*******      
        delay(schPara2);
     }
    }
else
   {
      for (int i2 = 0; i2 < 4; i2++)
      {
      
      
      //Stepper 2 Rechts Block
      
//*******
        if (arrayStepsLED5[i2] == 1)
        {
          digitalWrite(LED5, HIGH);
        }
        else
        {
          digitalWrite(LED5, LOW);
        }
//*******
        if (arrayStepsLED6[i2] == 1)
        {
          digitalWrite(LED6, HIGH);
        }
        else
        {
          digitalWrite(LED6, LOW);
        }
//*******
        if (arrayStepsLED7[i2] == 1)
        {
          digitalWrite(LED7, HIGH);
        }
        else
        {
          digitalWrite(LED7, LOW);
        }
//*******
        if (arrayStepsLED8[i2] == 1)
        {
          digitalWrite(LED8, HIGH);
        }
        else
        {
          digitalWrite(LED8, LOW);
        }
//*******
       delay(schPara2);
     }
    }
    digitalWrite(LED5, LOW);
    digitalWrite(LED6, LOW);
    digitalWrite(LED7, LOW);
    digitalWrite(LED8, LOW);
    if(i == repPara2-1 && schalt == 0){
       M2schaltPara2 = "aus";
       handle2OFF();}
   }
        
  } 

//**************************************************

       /* schalt = digitalRead(LED0); // SW-Bruecke. HE-Schalter (D0 INNPUT_PULLUP)
        
        if(schalt == 0 && M1HWPara1 == "true")
        {brueckAnal1 = analogRead(A0);delPara1 = brueckAnal1/10;
        if(delPara1 < 5){delPara1 = 4;delay(delPara1);}
        }else{delPara1 = schPara1;delay(delPara1);}
        
        if(schalt == 0 && M2HWPara2 == "true")
        {brueckAnal2 = analogRead(A0);delPara2 = brueckAnal2/10;
        if(delPara2 < 5){delPara2 = 4;delay(delPara2);}
        }else{delPara2 = schPara2;delay(delPara2);}
      */
}
