/* TPL Consulting, 2019 - HTML optimized RFID Read/Write
 *  
 * Initial Author: ryand1011 (https://github.com/ryand1011)
 *
 * Reads/Writes data written by a program such as "rfid_write_personal_data.ino"
*/

#include <SPI.h>
#include <MFRC522.h>

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>

#include "index.h"

#define RST_PIN 5   // Configured for NodeMCU LUA V2
#define SS_PIN  4   // Configured for NodeMCU LUA V2

#define LED0 16 //On board LED
#define LED3 0  //On board LED
#define LED8 15

//SSID Password WiFi router
const char* ssid = "your Name";
const char* password = "your Password";

ESP8266WebServer server(80); //Server on port 80

String firstName = "Vorname n.n.";
String lastName ="Nachname n.n.";
String strUID = "000.000.000.000";
String d0;
String d3;
String d8;
String persNr;

String wFirstName = "Vorname";
String wLastName ="Nachname";

String lesen = "off";
String schreiben = "off";

String wVn="Vorname";
String wNn = "Nachname";
String wUIDn1 = "000";
String wUIDn2 = "000";
String wUIDn3 = "000";
String wUIDn4 = "000";
String wDi1 = "0";
String wDi2 = "0";
String wDi3 = "0";
String wDate;
String wClock;
String wBut1 = "0";

               
void softwareReset(uint8_t prescaller) {
  wdt_enable(prescaller);
  while(1) {}
  delay(2000);
  wdt_disable();
}

void resetTrue(){
softwareReset(WDTO_1S);
}

// Browser öffnen

void handleRoot() {
 String s = MAIN_page; //HTML Inhalt
 server.send(200, "text/html", s);
}

//--- Schreibparameter ---

void handleWuid() {
lesen = "off";
wVn = server.arg("cardWf");
wNn = server.arg("cardWl");
wUIDn1 = server.arg("wuid1");
wUIDn2 = server.arg("wuid2");
wUIDn3 = server.arg("wuid3");
wUIDn4 = server.arg("wuid4");
wDi1 = server.arg("cb1");
wDi2 = server.arg("cb2");
wDi3 = server.arg("cb3");
wClock = server.arg("clock");
wDate = server.arg("datum");
wBut1 = server.arg("but1");

server.send(200, "text/html", wBut1);

lesen = "off";
schreiben = "on";
}

//--- Leseparameter ---

void handleFirstN() {
lesen = "on";
 server.send(200, "text/html", firstName);
}
 
void handleLastN() {
 server.send(200, "text/html", lastName);
}

void handleSuid() {
 server.send(200, "text/html", strUID);
}

void handleCB1() {
 server.send(200, "text/html", d0);
}
void handleCB2() {
 server.send(200, "text/html", d3);
}
void handleCB3() {
 server.send(200, "text/html", d8);
}
void handlePersNr() {
 server.send(200, "text/html", persNr);
}

MFRC522 mfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance


//*************//
void setup() {
  Serial.begin(115200);   // Initialize serial communications with the PC

  WiFi.begin(ssid, password); //WiFi router
  Serial.println("");

  //Warten auf Verbindung
  while (WiFi.status() != WL_CONNECTED) {
  delay(100);
  Serial.print(".");
  }


//Serieller Monitor
  Serial.println("");
  Serial.print("RFID Leser verbunden mit ");
  Serial.println(ssid);
  Serial.print("IP Adresse: ");
  Serial.println(WiFi.localIP());
  
  SPI.begin();            // Init SPI bus
  mfrc522.PCD_Init();     // Init MFRC522 card
  Serial.println("");
  Serial.println(F("Read/Write personal data on a MIFARE PICC:"));
  Serial.println("");

  pinMode(LED0,OUTPUT); 
  pinMode(LED3,OUTPUT); 
  pinMode(LED8,OUTPUT);


//HTML Routinenaufruf
  server.on("/", handleRoot);

  server.on("/readFname1", handleFirstN);
  server.on("/readLname1", handleLastN);

  server.on("/uidR", handleSuid);

  server.on("/uidW", handleWuid);

  server.on("/cb1R", handleCB1);
  server.on("/cb2R", handleCB2);
  server.on("/cb3R", handleCB3);

  server.on("/persNr1", handlePersNr);
  
  //Server starten
  server.begin();
}

//************//
void loop(){


//--- Karte lesen ---
if(lesen == "on" && schreiben == "off"){

// Prepare key - all keys are set to FFFFFFFFFFFFh
  MFRC522::MIFARE_Key key;
  for (byte i = 0; i < 6; i++) key.keyByte[i] = 0xFF;

  //Lesvariablen
  byte buffer[34];
  byte block;
  byte len;
  MFRC522::StatusCode status;
  firstName ="";
  lastName ="";
  //-----------------------

  // neue Karte?
  
  if ( ! mfrc522.PICC_IsNewCardPresent()) {
    return;
  }

  // Select one of the cards
  if ( ! mfrc522.PICC_ReadCardSerial()) {
    return;
  }
  
  Serial.println(F("**neue Karte:**"));

  //---------------------------------
  
  mfrc522.PICC_DumpDetailsToSerial(&(mfrc522.uid)); //dump some details about the card

  //-------------------------------

  len = 18;

  
  //--- GET FIRST NAME
  byte buffer1[18];
  block = 4;

  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, 4, &key, &(mfrc522.uid)); //line 834 of MFRC522.cpp file
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("Authentication failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";    
    return;
  }

  status = mfrc522.MIFARE_Read(block, buffer1, &len);
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("Reading failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";    
    return;
  }

  //PRINT FIRST NAME
  for (uint8_t i = 0; i < 16; i++)
  {
    if (buffer1[i] != 32)
    {
      const char firstNameorig = (buffer1[i]);
      firstName += firstNameorig;
    }
  }


  //--- GET LAST NAME
  byte buffer2[18];
  block = 1;

  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, 1, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("Authentication failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";    
    return;
  }

  status = mfrc522.MIFARE_Read(block, buffer2, &len);
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("Reading failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";    
    //return;
  }

  //PRINT LAST NAME
  for (uint8_t i = 0; i < 16; i++) {
      const char lastNameorig = (buffer2[i]);
      lastName += lastNameorig;
  }

  
  //--- GET D0, D3, D8 & Pers#

  byte buffer3[16];
  block = 8;

  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, 8, &key, &(mfrc522.uid)); //line 834
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("Authentication failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";    
    //return;
  }

  status = mfrc522.MIFARE_Read(block, buffer3, &len);
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("Reading failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }

  //PRINT D0, D3, D8 & Pers#
 
      const char d0orig = (buffer3[0]);
      d0 = d0orig;
      const char d3orig = (buffer3[1]);
      d3 = d3orig;
      const char d8orig = (buffer3[2]);
      d8 = d8orig;

      persNr = "";
  for (uint8_t i = 3; i < 16; i++) {
      const char persNrorig = (buffer3[i]);
      persNr += persNrorig;
  }

if(d0 !="0" && d0 !=""){digitalWrite(LED0,HIGH);}else{digitalWrite(LED0,LOW);}
if(d3 !="0" && d3 !=""){digitalWrite(LED3,HIGH);}else{digitalWrite(LED3,LOW);}
if(d8 !="0" && d8 !=""){digitalWrite(LED8,HIGH);}else{digitalWrite(LED8,LOW);}  
  //------------------------------------

  Serial.println(F("\n**Lesen ENDE**\n"));

  delay(500); //change value if you want to read cards faster
  
  const int readUIR = mfrc522.uid.uidByte[0] +  mfrc522.uid.uidByte[1] + mfrc522.uid.uidByte[2] + mfrc522.uid.uidByte[3];
  strUID = String(mfrc522.uid.uidByte[0])+ "." +String(mfrc522.uid.uidByte[1])+ "." +String(mfrc522.uid.uidByte[2])+ "." +String(mfrc522.uid.uidByte[3]);
  
  mfrc522.PICC_HaltA();
  mfrc522.PCD_StopCrypto1();
  lesen="off";
}



//--- Schreiben

if(schreiben == "on"){
  lesen="off";

  // Prepare key - all keys are set to FFFFFFFFFFFFh
  MFRC522::MIFARE_Key key;
  for (byte i = 0; i < 6; i++) key.keyByte[i] = 0xFF;

  // neue Karte?
  if ( ! mfrc522.PICC_IsNewCardPresent()) {
    return;
  }

  // Select one of the cards
  if ( ! mfrc522.PICC_ReadCardSerial()) {
    return;
  }

  Serial.print(F("Card UID:"));    //Dump UID
  for (byte i = 0; i < mfrc522.uid.size; i++) {
    Serial.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
    Serial.print(mfrc522.uid.uidByte[i], HEX);
  }
  Serial.print(F(" PICC type: "));   // Dump PICC type
  MFRC522::PICC_Type piccType = mfrc522.PICC_GetType(mfrc522.uid.sak);
  Serial.println(mfrc522.PICC_GetTypeName(piccType));

  byte buffer[34];
  byte block;
  MFRC522::StatusCode status;
  byte len;


  //--------------------------------
  // Daten Nachname
  wNn.toCharArray((char *)buffer, 30);
  len = wNn.length();
  for (byte i = len; i < 30; i++) buffer[i] = ' ';     // pad with spaces

  block = 1;

  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("PCD_Authenticate() failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }
  else Serial.println(F("PCD_Authenticate() success: "));

  // Write block
  status = mfrc522.MIFARE_Write(block, buffer, 16);
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("MIFARE_Write() failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }
  else Serial.println(F("MIFARE_Write() success: "));

  block = 2;

  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("PCD_Authenticate() failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }

  // Write block
  status = mfrc522.MIFARE_Write(block, &buffer[16], 16);
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("MIFARE_Write() failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }


  //-------------------------------
  // Daten Vorname
  
  wVn.toCharArray((char *)buffer, 20);
  len = wVn.length();
  for (byte i = len; i < 20; i++) buffer[i] = ' ';     // pad with spaces

  block = 4;

  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("PCD_Authenticate() failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }

  // Write block
  status = mfrc522.MIFARE_Write(block, buffer, 16);
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("MIFARE_Write() failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }
  else Serial.println(F("MIFARE_Write() success: "));

  block = 5;

  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("PCD_Authenticate() failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }

  // Write block
  status = mfrc522.MIFARE_Write(block, &buffer[16], 16);
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("MIFARE_Write() failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }
  else Serial.println(F("MIFARE_Write() success: "));


  //-------------------------------
  // Daten: D0, D3, D8 & Pers.#
  
  String wDges = wDi1 + wDi2 + wDi3 + wUIDn1 + "." + wUIDn2 + "." + wUIDn3;
  wDges.toCharArray((char *)buffer, 16);

  len = wDges.length();
  for (byte i = len; i < 16; i++) buffer[i] = ' ';     // pad with spaces

  block = 8;

  status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, block, &key, &(mfrc522.uid));
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("PCD_Authenticate() failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }

  // Write block
  status = mfrc522.MIFARE_Write(block, buffer, 16);
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("MIFARE_Write() failed: "));
    Serial.println(mfrc522.GetStatusCodeName(status));
    lesen="on";
    schreiben = "off";
    //return;
  }
  else Serial.println(F("MIFARE_Write() success: "));

  Serial.println(" ");
  Serial.println("ENDE Schreiben");
  mfrc522.PICC_HaltA();
  mfrc522.PCD_StopCrypto1();

    if(wBut1 == "1"){wBut1="0";}
    schreiben = "off";
    lesen="on";
}


//-----------------------------
  //Handle Client Anfrage
  server.handleClient();
}

